/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.xml.xpath.ext.spi;

import org.netbeans.modules.xml.schema.model.GlobalType;
import org.netbeans.modules.xml.xpath.ext.XPathSchemaContextHolder;
import org.netbeans.modules.xml.xpath.ext.schema.resolver.CastSchemaContext;

/**
 * A general type cast interface. It specifies a subject of cast and a
 * target schema type.
 * 
 * @author Vladimir Yaroslavskiy
 * @author Nikita Krjukov
 * @version 2009.01.26
 */
public interface XPathCast extends XPathSchemaContextHolder {

    /**
     * This method details semantics of the same method defined in the base interface.
     * @see XPathSchemaContextHolder
     * 
     * The result schema context is specially designed for XPathCast.
     * It holds full information about the casted component and the result cast type.
     * Using this method is preferred in comparison with the getPathExpression()
     * because it doesn't depend on namespace context. It means that a XPath expression
     * can be invalid beeing moved to another location because of namespace prefixes
     * can be different. But the schema contex will always be correct.
     * @return
     */
    CastSchemaContext getSchemaContext();

    /**
     * Returns a schema component to which an element of attribute is casted to.
     * Only GlobalType is supported now. 
     *
     * TODO: It probably worth modifying the result type to the
     * NamedComponentReference or
     * It will allow referencing to different global components.
     */
    GlobalType getType();
    
}
